package cn.minbb.job.model;

import cn.minbb.job.enumerate.Role;
import com.alibaba.fastjson.annotation.JSONField;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

/**
 * 用户基本信息
 */
@Data
@Entity
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@Table(name = "user",
        indexes = {
                @Index(name = "index_id", columnList = "id"),
                @Index(name = "index_username", columnList = "username"),
                @Index(name = "index_name", columnList = "name"),
                @Index(name = "index_company_id", columnList = "company_id")
        }
)
public class User extends Auditable implements UserDetails {

    @Column(name = "name", columnDefinition = "VARCHAR(32) COMMENT '姓名'")
    private String name;

    @Column(name = "username", nullable = false, columnDefinition = "VARCHAR(32) COMMENT '用户名'")
    private String username;

    @Column(name = "password", nullable = false, columnDefinition = "VARCHAR(256) COMMENT '密码'")
    private String password;

    @Column(name = "signature", columnDefinition = "VARCHAR(255) COMMENT '签名'")
    private String signature;

    @Column(name = "avatar", columnDefinition = "VARCHAR(255) COMMENT '头像'")
    private String avatar;

    @Column(name = "phone", columnDefinition = "VARCHAR(16) COMMENT '手机号'")
    private String phone;

    @Column(name = "mail", columnDefinition = "VARCHAR(255) COMMENT '邮箱'")
    private String mail;

    @Column(name = "qq", columnDefinition = "VARCHAR(16) COMMENT 'QQ'")
    private String qq;

    @Column(name = "is_account_non_expired", nullable = false, columnDefinition = "TINYINT DEFAULT '1' COMMENT '帐户未过期'")
    private Boolean isAccountNonExpired;

    @Column(name = "is_account_non_locked", nullable = false, columnDefinition = "TINYINT DEFAULT '1' COMMENT '帐户未锁定'")
    private Boolean isAccountNonLocked;

    @Column(name = "is_credentials_non_expired", nullable = false, columnDefinition = "TINYINT DEFAULT '1' COMMENT '凭据未过期'")
    private Boolean isCredentialsNonExpired;

    @Column(name = "is_enabled", nullable = false, columnDefinition = "TINYINT DEFAULT '1' COMMENT '帐户已启用'")
    private Boolean isEnabled;

    @ManyToMany(cascade = {CascadeType.REFRESH}, fetch = FetchType.EAGER)
    @JoinTable(name = "user_user_role",
            joinColumns = {@JoinColumn(name = "user_id", referencedColumnName = "id")},
            inverseJoinColumns = {@JoinColumn(name = "user_role_id", referencedColumnName = "id")}
    )
    private Set<UserRole> userRoleSet;

    @OneToOne(cascade = CascadeType.ALL, fetch = FetchType.EAGER, optional = true)
    @JoinColumn(name = "company_id", columnDefinition = "INT COMMENT '企业ID'")
    private Company company;

    @JSONField(serialize = false)
    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        List<GrantedAuthority> grantedAuthorityList = new ArrayList<>();
        for (UserRole userRole : this.getUserRoleSet()) {
            grantedAuthorityList.add(new SimpleGrantedAuthority("ROLE_" + userRole.getRole().name()));
        }
        return grantedAuthorityList;
    }

    @JSONField(serialize = false)
    @Override
    public boolean isAccountNonExpired() {
        return this.isAccountNonExpired;
    }

    @JSONField(serialize = false)
    @Override
    public boolean isAccountNonLocked() {
        return this.isAccountNonLocked;
    }

    @JSONField(serialize = false)
    @Override
    public boolean isCredentialsNonExpired() {
        return this.isCredentialsNonExpired;
    }

    @JSONField(serialize = false)
    @Override
    public boolean isEnabled() {
        return this.isEnabled;
    }

    public boolean hasRole(Role role) {
        for (UserRole userRole : getUserRoleSet()) {
            if (userRole.getRole().equals(role)) {
                return true;
            }
        }
        return false;
    }

    public User(String name, String username, String password) {
        this.name = name;
        this.username = username;
        this.password = password;
        this.isAccountNonExpired = Boolean.TRUE;
        this.isAccountNonLocked = Boolean.TRUE;
        this.isCredentialsNonExpired = Boolean.TRUE;
        this.isEnabled = Boolean.TRUE;
    }
}
