package cn.minbb.job.system;

import cn.minbb.job.model.User;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.context.SecurityContextImpl;
import org.springframework.security.web.authentication.WebAuthenticationDetails;
import org.springframework.security.web.authentication.preauth.PreAuthenticatedAuthenticationToken;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

public class UserSession {

    private UserSession() {
    }

    /**
     * 用户认证 - 获取当前会话中已登录的用户
     *
     * @return 已登录用户
     */
    public static User getUserAuthentication() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (null != authentication) {
            Object object = authentication.getPrincipal();
            if (object instanceof User) {
                return (User) object;
            }
        }
        return null;
    }

    /**
     * 更新用户认证信息，此操作会导致原会话失效，即需要用户重新登录
     *
     * @param request HttpServletRequest
     * @param user    用户
     */
    public static void updateUserAuthentication(HttpServletRequest request, User user) {
        // 1.从 HttpServletRequest 中获取 SecurityContextImpl 对象
        SecurityContextImpl securityContextImpl = (SecurityContextImpl) request.getSession().getAttribute("SPRING_SECURITY_CONTEXT");
        // 2.从 SecurityContextImpl 中获取 Authentication 对象
        Authentication authentication = securityContextImpl.getAuthentication();
        // 3.初始化 UsernamePasswordAuthenticationToken 实例，参数 user 就是我们要更新的用户信息
        UsernamePasswordAuthenticationToken auth = new UsernamePasswordAuthenticationToken(user, authentication.getCredentials());
        auth.setDetails(authentication.getDetails());
        // 4.重新设置 SecurityContextImpl 对象的 Authentication
        securityContextImpl.setAuthentication(auth);
        // SecurityContextHolder.getContext().setAuthentication(auth);
    }

    /**
     * 更新用户会话信息，此操作不会导致原会话失效，不需要用户重新登录
     *
     * @param request HttpServletRequest
     * @param user    包含认证凭证（即密码）和 Authorities 的用户
     */
    public static void updateUserSession(HttpServletRequest request, User user) {
        // 根据 userDetails 构建新的 Authentication
        // 这里使用 PreAuthenticatedAuthenticationToken
        // 也可以用其他 token, 如 UsernamePasswordAuthenticationToken               
        PreAuthenticatedAuthenticationToken authentication =
                new PreAuthenticatedAuthenticationToken(user, user.getPassword(), user.getAuthorities());
        // 设置 authentication 中 details
        authentication.setDetails(new WebAuthenticationDetails(request));
        // 存放 authentication 到 SecurityContextHolder
        SecurityContextHolder.getContext().setAuthentication(authentication);
        HttpSession session = request.getSession(true);
        // 在 session 中存放 security context,方便同一个 session 中控制用户的其他操作
        session.setAttribute("SPRING_SECURITY_CONTEXT", SecurityContextHolder.getContext());
    }
}
