package cn.minbb.job.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

public class DateUtil {

    public final static String DATE_FORMAT = "yyyy-MM-dd";
    public final static String DATE_TIME_FORMAT = "yyyy-MM-dd HH:mm:ss";

    /**
     * 字符串格式转日期
     *
     * @param date yyyy-MM-dd HH:mm:ss
     */
    public static Date stringToDate(String date) {
        try {
            return new SimpleDateFormat(DATE_TIME_FORMAT).parse(date);
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 日期格式转字符串
     *
     * @return yyyy-MM-dd HH:mm:ss
     */
    public static String dateToString(Date date) {
        return null == date ? "" : new SimpleDateFormat(DATE_TIME_FORMAT).format(date);
    }

    /**
     * 计算两个日期差的天数
     *
     * @param startTime 开始日期
     * @param endTime   结束日期
     * @return 相差天数（整数天）
     */
    public static int daysBetween(Date startTime, Date endTime) {
        Calendar calendarStart = Calendar.getInstance();
        calendarStart.setTime(endTime);
        calendarStart.set(Calendar.HOUR_OF_DAY, 0);
        calendarStart.set(Calendar.MINUTE, 0);
        calendarStart.set(Calendar.SECOND, 0);
        Calendar calendarEnd = Calendar.getInstance();
        calendarEnd.setTime(startTime);
        calendarEnd.set(Calendar.HOUR_OF_DAY, 0);
        calendarEnd.set(Calendar.MINUTE, 0);
        calendarEnd.set(Calendar.SECOND, 0);
        long millisStart = calendarStart.getTimeInMillis();
        long millisEnd = calendarEnd.getTimeInMillis();
        long millisAbs = Math.abs(millisStart - millisEnd);
        return (int) (millisAbs / (1000 * 86400));
    }

    /**
     * 对指定的日期增加或减少指定的秒
     *
     * @param date    日期
     * @param seconds 秒
     * @return 日期
     */
    public static Date addOrMinusSeconds(Date date, int seconds) {
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTime(date);
        gregorianCalendar.add(Calendar.SECOND, seconds);
        return gregorianCalendar.getTime();
    }

    /**
     * 对指定的日期增加或减少指定的分钟
     *
     * @param date    日期
     * @param minutes 分钟
     * @return 日期
     */
    public static Date addOrMinusMinutes(Date date, int minutes) {
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTime(date);
        gregorianCalendar.add(Calendar.MINUTE, minutes);
        return gregorianCalendar.getTime();
    }

    /**
     * 对指定的日期增加或减少指定的小时数
     *
     * @param date  日期
     * @param hours 小时
     * @return 日期
     */
    public static Date addOrMinusHours(Date date, int hours) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.HOUR_OF_DAY, hours);
        return calendar.getTime();
    }

    /**
     * 对指定的日期增加或减少指定的天数
     *
     * @param date 日期
     * @param days 天数
     * @return 日期
     */
    public static Date addOrMinusDays(Date date, int days) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, days);
        return calendar.getTime();
    }

    /**
     * 对指定的日期增加或减少指定的月数
     *
     * @param date   日期
     * @param months 月数
     * @return 日期
     */
    public static Date addOrMinusMonths(Date date, int months) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, months);
        return calendar.getTime();
    }

    /**
     * 获取当前日期的字符串格式
     *
     * @return 当前日期字符串 yyyy-MM-dd HH:mm:ss
     */
    public static String getStringDateOfCurrent() {
        return dateToString(new Date());
    }

    /**
     * 获取日期，去除时间
     *
     * @return 日期 yyyy-MM-dd 00:00:00
     */
    public static Date getDateOnlyByDateTime(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取当前日期的前一天
     *
     * @return 当前日期的前一天
     */
    public static Date getBeforeDayOfCurrent() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        return calendar.getTime();
    }

    /**
     * 获取当前日期的后一天
     *
     * @return 当前日期的后一天
     */
    public static Date getNextDayOfCurrent() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, 1);
        return calendar.getTime();
    }

    /**
     * 获取当月的第一天
     *
     * @return 当月的第一天
     */
    public static Date getFirstDateOfMonth(Date date) {
        if (null == date) date = new Date();
        Calendar calendar = Calendar.getInstance(Locale.CHINA);
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, 0);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取当前月的最后一天
     *
     * @return 当前月的最后一天
     */
    public static Date getLastDateOfMonth(Date date) {
        if (null == date) date = new Date();
        Calendar calendar = Calendar.getInstance(Locale.CHINA);
        calendar.setTime(date);
        // 最后一天
        int lastDate = calendar.getActualMaximum(Calendar.DATE);
        calendar.set(Calendar.DATE, lastDate);
        return calendar.getTime();

    }

    public static void main(String[] args) {
        System.out.println(new SimpleDateFormat(DATE_TIME_FORMAT).format(getFirstDateOfMonth(null)));
        System.err.println(dateToString(getDateOnlyByDateTime(new Date())));
        System.err.println(System.currentTimeMillis());
    }
}
