package cn.minbb.job.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.log4j.Log4j2;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;

@Component
@Log4j2
public class IPUtil {

    private static final String UNKNOWN = "unknown";

    private IPUtil() {
    }

    /**
     * 根据 HttpServletRequest 请求获取客户端真实 IP 地址
     *
     * @param request HttpServletRequest
     * @return 真实 IP 地址串
     */
    public String getIpAddress(HttpServletRequest request) {
        String ipAddress;
        try {
            // x-forwarded-for
            ipAddress = request.getHeader("X-Forwarded-For");
            if (ipAddress == null || ipAddress.length() == 0 || UNKNOWN.equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("Proxy-Client-IP");
            }
            if (ipAddress == null || ipAddress.length() == 0 || UNKNOWN.equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("WL-Proxy-Client-IP");
            }
            if (ipAddress == null || ipAddress.length() == 0 || UNKNOWN.equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("HTTP_CLIENT_IP");
            }
            if (ipAddress == null || ipAddress.length() == 0 || UNKNOWN.equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("HTTP_X_FORWARDED_FOR");
            }
            if (ipAddress == null || ipAddress.length() == 0 || UNKNOWN.equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getRemoteAddr();
                if (ipAddress.equals("127.0.0.1")) {
                    // 根据网卡取本机配置的 IP
                    ipAddress = InetAddress.getLocalHost().getHostAddress();
                }
            }
            // 对于通过多个代理的情况，第一个 IP 为客户端真实 IP,多个 IP 按照','分割
            if (ipAddress != null && ipAddress.length() > 15) {
                // "***.***.***.***".length() = 15
                int index = ipAddress.indexOf(',');
                if (index > 0) {
                    ipAddress = ipAddress.substring(0, index);
                }
            }
        } catch (Exception e) {
            ipAddress = "";
            log.info("{} - {}", e.getCause(), e.getMessage());
        }
        return ipAddress;
    }

    /**
     * 太平洋网络 IP 地址查询接口
     * 不存在时间查询间隔
     *
     * @param ip IP 地址
     * @return 位置信息和运营商
     */
    public String getIPLocationByTPY(String ip) {
        String address = null;
        try {
            String url = "http://whois.pconline.com.cn/ipJson.jsp?ip={1}&json=true";
            ResponseEntity<String> entity = new RestTemplate().getForEntity(url, String.class, ip);
            if (entity.getStatusCode() == HttpStatus.OK) {
                String body = entity.getBody();
                JSONObject json = JSON.parseObject(null == body ? "{}" : body.trim());
                address = json.getString("addr");
            }
        } catch (Exception e) {
            log.error("查询 IP 地址异常，{}", e.getMessage());
        }
        return address;
    }
}
